#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

float rand1(idum)
long *idum;
{  int j;
   long k;
   static long iy =0;
   static long iv[NTAB];
   float temp;
   float ret;

   if( *idum<=0 || !iy)
   {  
      if( -(*idum)<1) *idum=1;
      else *idum = -(*idum);

      for(j=NTAB+7;j>=0;j--)
      {  k=(*idum)/IQ;
	 *idum=IA*(*idum-k*IQ)-IR*k;
	 if(*idum<0) *idum+=IM;
	 if(j<NTAB) iv[j]=*idum;
      }/* end of for */

      iy=iv[0];

   }/* end of if */

   k=(*idum)/IQ;
   *idum=IA*(*idum-k*IQ)-IR*k;
   if(*idum<0) *idum+=IM;
   j=iy/NDIV;
   iy=iv[j];
   iv[j]=*idum;

   if((temp=AM*iy)>RNMX) ret=RNMX;
   else ret = temp;

   return ret;

}/* end of rand1() */

float gasdev(idum)
long *idum;
{  static int iset=0;
   static float gset;
   float fac, rsq, v1, v2, ret;

   if(iset == 0)
   {  do 
      {  v1=2.0*rand1(idum)-1.0;
	 v2=2.0*rand1(idum)-1.0;
	 rsq = v1*v1+v2*v2;
      } while (rsq >= 1.0 || rsq == 0.0 );
      fac = sqrt(-2.0*log(rsq)/rsq);
      gset=v1*fac;
      iset=1;
      ret = v2*fac;
   } 
   else {  iset = 0 ;
	   ret = gset;
        }

   return ret;

}/* end of gasdev() */

float rgauss(idum,mean,standartdeviation)
long *idum;
float mean,standartdeviation;
{  float ret = (standartdeviation*standartdeviation*gasdev(idum)+mean);
   if(fabs(ret)>standartdeviation) ret = remainder(ret,standartdeviation);
   return ret;
}/* end of rgauss() */

